import os

from azure.appconfiguration import AzureAppConfigurationClient
from azure.keyvault.secrets import SecretClient
from azure.core.exceptions import ResourceNotFoundError
from azure.identity import DefaultAzureCredential

from environs import Env,EnvError

def seek_config(resolvers=None):
    if resolvers is None:
        resolvers = list()
    def return_config(key):
        for res in resolvers:
            try:
                return res(key)
            except KeyError:
                continue
        raise KeyError
    return return_config

dict_resolver = lambda dictionary: lambda k: dictionary[k]

def azure_appconfig_resolver(connection_string):

    client = AzureAppConfigurationClient.from_connection_string(connection_string)

    def return_config(k):
        try:
            return client.get_configuration_setting(k).value
        except ResourceNotFoundError as rnfe:
            raise KeyError from rnfe

    return return_config

def azure_keyvault_secret_resolver(keyvault_url,credentials=None):

    if credentials is None:
        credentials = DefaultAzureCredential() 

    client = SecretClient(keyvault_url,credentials)

    def return_config(k):
        fixed_key = k.replace("_","-").lower() # keyvault does not allow _
        try:
            return client.get_secret(fixed_key).value
        except ResourceNotFoundError as rnfe:
            raise KeyError from rnfe

    return return_config

def environs_resolver():

    env = Env()
    env.read_env()

    def return_config(k):
        try:
            return env(k)
        except EnvError as ee:
            raise KeyError from ee 

    return return_config

def views_config(keyvault_url,credentials=None):
    environs = environs_resolver()
    secrets = azure_keyvault_secret_resolver(keyvault_url,credentials)
    appconfig = azure_appconfig_resolver(secrets("appconfig-connection-string"))
    return seek_config([environs,appconfig,secrets])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fitin']

package_data = \
{'': ['*']}

install_requires = \
['azure-appconfiguration>=1.1.1,<2.0.0',
 'azure-identity>=1.5.0,<2.0.0',
 'azure-keyvault-secrets>=4.2.0,<5.0.0',
 'environs>=9.3.1,<10.0.0']

setup_kwargs = {
    'name': 'fitin',
    'version': '0.2.0',
    'description': 'Simple config management, making it easy to write services that fit in.',
    'long_description': None,
    'author': 'peder2911',
    'author_email': 'pglandsverk@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
